import numpy as np
import cv2
from pathlib import Path


def loadvideo(filename: str):
    capture = cv2.VideoCapture(str(filename))

    frame_count = int(capture.get(cv2.CAP_PROP_FRAME_COUNT))
    frame_width = int(capture.get(cv2.CAP_PROP_FRAME_WIDTH))
    frame_height = int(capture.get(cv2.CAP_PROP_FRAME_HEIGHT))
    frame_rate = int(capture.get(cv2.CAP_PROP_FPS))

    v = np.zeros((frame_count, frame_height, frame_width), np.uint8)

    for count in range(frame_count):
        ret, frame = capture.read()
        if not ret:
            raise ValueError("Failed to load frame #{} of {}.".format(count, filename))

        frame = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
        v[count] = frame

    return (v, frame_rate, frame_count, frame_height, frame_width)


def savevideo(outFile, array, fps):
    f, height, width = array.shape
    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    out = cv2.VideoWriter(str(outFile), fourcc, fps, (width, height), False)
    for i in range(f):
        out.write(array[i, :,:])
    out.release()


def countUniquePixels(a):
    return len(np.unique(a))


def countUniquePixelsAlongFrames(vid):
    f, height, width = vid.shape
    u = np.zeros((height, width), np.uint8)
    for i in range(height):
        u[i] = np.apply_along_axis(countUniquePixels, 0, vid[:,i,:])

    return u


def findEdges(x, thresh=0.1):
    # find the right
    right = len(x)
    a = x[int(len(x)/2):-1]
    b, = np.where( a <= thresh )
    if len(b)>0:
        right = int(len(x)/2)+b[0]

    # find the left
    left = 0
    a = np.flip(x[0:int(len(x)/2)])
    b, = np.where( a <= thresh )
    if len(b)>0:
        left = int(len(x)/2)-b[0]

    return (left,right)


def crop(inputfile, outputfile, thresh=0.1):
  vid, fps, f, height, width = loadvideo(inputfile)

  u = countUniquePixelsAlongFrames(vid)
  u_avg = u/f

  maxW = np.max(u_avg, axis=0)
  left,right = findEdges(maxW, thresh=thresh)
  maxH = np.max(u_avg, axis=1)
  top,bottom = findEdges(maxH, thresh=thresh)

  cropped = vid[:,top:bottom,left:right]
  savevideo(outputfile, cropped, fps)



def applyMask(vid, mask):
    y = np.zeros_like(vid)
    f, height, width = vid.shape
    for i in range(f):
        y[i][mask] = vid[i][mask]

    return(y)









import numpy as np
import cv2
from pathlib import Path

import fire

from sonocrop import vid

def crop(input_file, output_file, thresh=0.1):
  """
  Finds the center of an ultrasound and crops out the static borders
  """

  vid.crop(input_file, output_file, thresh=thresh)


def edges(input_file, thresh=0.1):
  v, fps, f, height, width = vid.loadvideo(input_file)

  u = vid.countUniquePixelsAlongFrames(v)
  u_avg = u/f

  maxW = np.max(u_avg, axis=0)
  left,right = vid.findEdges(maxW, thresh=thresh)
  maxH = np.max(u_avg, axis=1)
  top,bottom = vid.findEdges(maxH, thresh=thresh)

  return (f'{left},{right},{top},{bottom}')


def mask(input_file, output_file, thresh=0.1):
  v, fps, f, height, width = vid.loadvideo(input_file)
  u = vid.countUniquePixelsAlongFrames(v)
  u_avg = u/f
  mask = u_avg > thresh
  y = vid.applyMask(v, mask)
  vid.savevideo(output_file, y, fps)
  return ('Done')


def main():
  fire.Fire()

if __name__ == '__main__':
  main()



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

version = {}
with open(path.join(here, "sonocrop", "__version__.py")) as f:
    exec(f.read(), version)

setup(

  name='sonocrop',

  version=version["__version__"],

  description='Prepare ultrasound videos for machine learning-- crop and remove static clutter from ultrasound video.',

  long_description=long_description,

  long_description_content_type='text/markdown',

  url='https://github.com/davycro/sonocrop',

  author='David Crockett, MD',

  author_email='davycro1@gmail.com',

  license = "Apache Software License 2.0",


  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Healthcare Industry',
    'Topic :: Multimedia :: Video',
    'Topic :: Scientific/Engineering :: Medical Science Apps.',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8'
  ],

  keywords='ultrasound bedside ultrasound pocus ffmpeg opencv echo cardiac',

  packages=find_packages(),

  python_requires='>=3.6',

  install_requires=[
    'numpy',
    'opencv-python',
    'fire',
  ],

  entry_points={  # Optional
      'console_scripts': [
          'sonocrop=sonocrop.cli:main',
      ],
  },

)
